import json
import os
import time
import subprocess
import re
import tlid

import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))


def json_to_dict(json_str):
    return json.loads(json_str)

# IMPORT from CM app.py, by J.Guillaume Isabelle, 2024-06-20
def postprocess_abc(text, conversation_id="test",musescore_bin = "musescore3",use_tlider=True,workdir="tmp",prefix="",quiet=False,sc_namespace_suffix=None):
    abc2midiExecutable = "abc2midi"
    #check if musescore_bin is an executable, else use "musescore"
    #if not os.path.isfile(musescore_bin):
    #    musescore_bin = "musescore"
    
    if workdir == "":
      workdir="."
    if conversation_id != "": 
      os.makedirs(f"{workdir}/{conversation_id}", exist_ok=True)
    extracted_abc_notation_from_text = extract_abc_from_text(text)
    if not quiet:
      print(f'extract abc block: {extracted_abc_notation_from_text}')
    if extracted_abc_notation_from_text:
        
        
        sc_namespace_suffix = newsc_namespace_suffix(prefix,use_tlider=use_tlider) if not sc_namespace_suffix else sc_namespace_suffix # So we can define it in the function call
        
        # Write the ABC text to a temporary file
        res_abc_filepath = mkns_filepath(conversation_id, workdir, sc_namespace_suffix,"abc")
        with open(res_abc_filepath, "w") as abc_file:
            abc_file.write(extracted_abc_notation_from_text[0])
        
        # Convert abc notation to midi
        res_midi_filepath = mkns_filepath(conversation_id, workdir, sc_namespace_suffix,"mid")
        _convert_abc_2_midi(res_abc_filepath, res_midi_filepath)
        
        # Convert abc notation to SVG
        res_musicsheet_svg_filepath = mkns_filepath(conversation_id, workdir, sc_namespace_suffix,"svg")
        res_audio_filepath = mkns_filepath(conversation_id, workdir, sc_namespace_suffix,"mp3")
        
        capture_output_of_command = True if not quiet else False
        
        _convert_midi_2_score(res_midi_filepath, res_musicsheet_svg_filepath, capture_output_of_command,musescore_bin=musescore_bin)
        _convert_midi_to_mp3(res_midi_filepath, res_audio_filepath,musescore_bin=musescore_bin)
        
        # Fix the SVG file path (from ChatMusician, why do they do that ?)
        res_musicsheet_svg_filepath_fixed = mkns_filepath(conversation_id, workdir, sc_namespace_suffix,"svg","-1")
        
        if not quiet:
          print("-----------------------------")
          print(" FILES CREATED: ")
          print(res_musicsheet_svg_filepath_fixed)
          print(res_audio_filepath)
          print(res_abc_filepath)
          print(res_midi_filepath)
        #print_postprocess_abc_result_as_markdown(res_musicsheet_svg_filepath_fixed, res_audio_filepath,res_abc_filepath,render=False)
        return res_musicsheet_svg_filepath_fixed, res_audio_filepath,res_abc_filepath,res_midi_filepath
    else:
        return None, None 

def extract_abc_from_text(text):
    abc_pattern_extractor_str = r'(X:\d+\n(?:[^\n]*\n)+)'
    extracted_abc_notation_from_text = re.findall(abc_pattern_extractor_str, text+'\n')
    return extracted_abc_notation_from_text

def _convert_midi_to_mp3(res_midi_filepath, res_audio_filepath,musescore_bin = "musescore3", capture_output_of_command=False):
  subprocess.run([musescore_bin,"-o", res_audio_filepath, res_midi_filepath],capture_output=capture_output_of_command, text=True)

def _convert_midi_2_score(res_midi_filepath, res_musicsheet_svg_filepath, capture_output_of_command=False,musescore_bin = "musescore3"):
  subprocess.run([musescore_bin, "-o", res_musicsheet_svg_filepath, res_midi_filepath], capture_output=capture_output_of_command, text=True)

def _convert_abc_2_midi(res_abc_filepath, res_midi_filepath,abc2midiExecutable = "abc2midi"):
  subprocess.run([abc2midiExecutable, str(res_abc_filepath), "-o", res_midi_filepath])

def newsc_namespace_suffix(prefix, use_tlider=True):
  ts_or_tlid = _newts_suffix(use_tlider)
  _prefix = prefix + "_" if prefix else ""
  return _prefix + str(ts_or_tlid)

def _newts_suffix(use_tlider):
  return time.time() if not use_tlider else tlid.get_tlid()

def mkns_filepath(conversation_id, workdir, ts_namespace,ext,fn_suffix=""):
  return f"{workdir}/{conversation_id}/{ts_namespace}{fn_suffix}.{ext}"

def generate_markdown_output(res_musicsheet_svg_filepath_fixed, res_audio_filepath, res_abc_filepath,res_midi_filepath, line_prefix="* ",end_of_line_char = "\n",b4_musicscore="  "):
  if res_musicsheet_svg_filepath_fixed:
    abc_source = f"{line_prefix}[ABC source:]({res_abc_filepath})"
    midi_source=f"{line_prefix}[MIDI source:]({res_midi_filepath})"
    audio = f"{line_prefix}[Audio]({res_audio_filepath})"
    music_sheet = f"{b4_musicscore}![Music Sheet]({res_musicsheet_svg_filepath_fixed})"
    
    return abc_source + end_of_line_char+ midi_source + end_of_line_char+ audio + end_of_line_char +  music_sheet
  else:
    return "No ABC block found in the input text.",

def print_markdown_output(markdown_output):
  for line in markdown_output:
    print(line)

def print_postprocess_abc_result_as_markdown(res_musicsheet_svg_filepath_fixed, res_audio_filepath,res_abc_filepath,res_midi_filepath,line_prefix="* ",render=True):
  markdown_output = generate_markdown_output(res_musicsheet_svg_filepath_fixed, res_audio_filepath, res_abc_filepath,res_midi_filepath, line_prefix)
  print_markdown_output(markdown_output) if not render else print_markdown_output_in_jupyter(markdown_output)


def print_markdown_output_in_jupyter(markdown_output):
  from IPython.display import display, Markdown
  #markdown = "\n".join(markdown_output)
  display(Markdown(markdown_output))

def play_midi_in_notebook(res_mid_filepath):
  from music21 import midi
  mf = midi.MidiFile()
  path=res_mid_filepath
  mf.open(path) # path='abc.midi'
  mf.read()
  mf.close()
  s = midi.translate.midiFileToStream(mf)
  s.makeNotation(inPlace=True)
  s.secondsMap
  s.show('midi')


def play_mp3_in_notebook(res_audio_filepath):
  from IPython.display import Audio
  return Audio(res_audio_filepath,autoplay=True)


def get_generated_text_from_json(filepath):
  with open(filepath, 'r') as json_file:
    data = json.load(json_file)
    return data[0].get('generated_text', None)



# Get all json files in the directory './data/' + workdir
def get_json_files_list(workdir,exclude_input_prompts=True):
    json_files = []
    working_directory_path = './' + workdir
    print('Working directory path: ' + working_directory_path)
    for file in os.listdir(working_directory_path):
        if file.endswith('.json'):
            json_files.append(file)
        if file.endswith('-input_prompts.json') and exclude_input_prompts:
            json_files.remove(file)
    return json_files


def save_as_json(output,workdir,scprefix,sc_namespace_suffix,suffix=""):
  outdir=f"{workdir}"
  os.makedirs(outdir, exist_ok=True)
  filepath=f"{workdir}/{scprefix}{sc_namespace_suffix}{suffix}.json"
  with open(filepath, 'w') as f:
    json.dump(output, f, indent=4)
    print("written: ",filepath)
    return filepath
  return None


def save_as_json_to_filename(output,filepath):
  _filepath=f"{filepath}.json"
  fixed_filepath = _filepath.replace(".json.json",".json")
  with open(fixed_filepath, 'w') as f:
    json.dump(output, f, indent=4)
    print("written: ",fixed_filepath)
    return fixed_filepath
  return None



import requests
def run_inference_query(payload,api_url,headers = {
	"Accept" : "application/json",
	"Content-Type": "application/json" 
}):
  response = requests.post(api_url, headers=headers, json=payload)
  return response.json()



import unittest
from jgcmlib.module import Module

class TestModule(unittest.TestCase):
    def test_something(self):
        # Add your test case here
        pass

if __name__ == '__main__':
    unittest.main()


# class Module:
#     def __init__(self):
#         # Initialize the module
#         pass

#     def method1(self):
#         # Implement method 1
#         pass

#     def method2(self):
#         # Implement method 2
#         pass

#     # Add more methods, properties, or constants as needed



import sys
import os
import argparse
from time import sleep
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
try:
  import jgcmlib as jcm
except:
  import __init__ as jcm
import json
import subprocess

def pto__convert_midi_2_score(filepath, musescore_bin = "musescore3"):
  filebase = os.path.basename(filepath)
  output_dir=os.path.dirname(filepath)
  res_musicsheet_svg_filepath=os.path.join(output_dir, filebase.replace(".mid",".svg"))
  res_musicsheet_svg_filepath=res_musicsheet_svg_filepath+".svg"
  res_musicsheet_svg_filepath=res_musicsheet_svg_filepath.replace(".svg.svg",".svg")
  
  # Convert midi to musicsheet
  
  try:
    jcm._convert_midi_2_score(filepath, res_musicsheet_svg_filepath,musescore_bin=musescore_bin)
  except subprocess.CalledProcessError as e:
    print("Error: Could not convert the midi file to musicsheet. ", e)
    return
  return res_musicsheet_svg_filepath
 
def pto_post_just_an_abc_file(filepath,musescore_bin = "musescore3",abc2midiExecutable = "abc2midi"):
  
  filebase = os.path.basename(filepath)
  output_dir=os.path.dirname(filepath)
  res_midi_filepath=os.path.join(output_dir, filebase.replace(".abc",".mid"))
  res_audio_filepath=os.path.join(output_dir, filebase.replace(".abc",".mp3"))
  res_musicsheet_svg_filepath=os.path.join(output_dir, filebase.replace(".abc",".svg"))
  
  jcm._convert_abc_2_midi(filepath, res_midi_filepath,abc2midiExecutable=abc2midiExecutable)
  try:
    jcm._convert_midi_to_mp3(res_midi_filepath, res_audio_filepath,musescore_bin=musescore_bin)
  except:
    musescore_bin="musescore"
    try:
      jcm._convert_midi_to_mp3(res_midi_filepath, res_audio_filepath,musescore_bin=musescore_bin)
    except:
      print("Error: Could not convert the midi file to mp3. Something with musescore is not right.")
      sleep(2)
      return
      #raise Exception("Error: Could not convert the midi file to mp3. Something with musescore is not right.")
  jcm._convert_midi_2_score(filepath, res_musicsheet_svg_filepath,musescore_bin=musescore_bin)
  return res_musicsheet_svg_filepath, res_audio_filepath, res_midi_filepath


def main_mid2score():
  args = create_arg_parser(argparse)
  res_musicsheet_svg_filepath = pto__convert_midi_2_score(args.inputfile,musescore_bin=args.musescore_bin)
  print("res_musicsheet_svg_filepath: ", res_musicsheet_svg_filepath)

def main():
  args = create_arg_parser(argparse)
  
  abc_filename = args.inputfile
  #Check if we were given an .abc or a .json
  if not args.inputfile.endswith(".abc"):
    if args.inputfile.endswith(".json"):
      try:
          
        with open(args.inputfile, 'r') as f:
          data = json.load(f)
          if isinstance(data, list):
            generated_text = data[0]['generated_text']
          else:
            generated_text = data['generated_text']
          print("generated_text: ", generated_text)
          txt_filename=args.inputfile.replace(".json",".txt")
          with open(txt_filename, "w") as txt_file:
            txt_file.write(generated_text)
          
          abc_extracted=jcm.extract_abc_from_text(generated_text)
          #if abc_extracted list is empty  exit program
          if len(abc_extracted) == 0 or not abc_extracted:
            print("Error: Could not extract the abc notation from the json file.")
            print(data)
            exit(1)
          print("abc extracted:",abc_extracted)
          abc_filename=args.inputfile.replace(".json",".abc")
          try:
            with open(abc_filename, "w") as abc_file:
              abc_file.write(abc_extracted[0])
          except:
            print("Error: Could not write the abc file.  There might just did not have any abc notation in the json.")
            exit(1)
      except:
        print("Error: Could not read the json file.")
        return
          
           
    #print("Only .abc files are supported for now.")
    #return
  
  res_musicsheet_svg_filepath, res_audio_filepath, res_midi_filepath =  pto_post_just_an_abc_file(abc_filename,musescore_bin=args.musescore_bin,abc2midiExecutable=args.abc2midi_bin)
  print("res_musicsheet_svg_filepath: ", res_musicsheet_svg_filepath)
  print("res_audio_filepath: ", res_audio_filepath)
  print("res_midi_filepath: ", res_midi_filepath)

def create_arg_parser(argparse):
    parser = argparse.ArgumentParser(description='Convert an ABC file to a MIDI file, a MP3 file and a SVG file')
    parser.add_argument('inputfile', type=str, help='The ABC or JSON file to convert')
    parser.add_argument('--musescore-bin', type=str, default="musescore3", help='The path to the musescore binary (default: musescore3)')
    parser.add_argument('--abc2midi-bin', type=str, default="abc2midi", help='The path to the abc2midi binary. (default: abc2midi)')
    args = parser.parse_args()
    return args

if __name__ == "__main__":
  main()


from setuptools import setup, find_packages

setup(
    name='jgcmlib',
    version='1.0.41',
    author='J.Guillaume D-Isabelle',
    description='A Python module jgcmlib',
    url='https://github.com/jgwill/jgcmlib',
    packages=find_packages(),
    install_requires=[
        "tlid",
        "requests",
        "music21",
        "ipython",
    ],
    entry_points={
        'console_scripts': [
            "jgabcli = jgcmlib.jgabcli:main"
        ]
    },
)